from enum import Enum

class NTag():
    class Types(Enum):
        ARTIST      = 1
        CATEGORY    = 2
        CHARACTER   = 3
        GROUP       = 4
        LANGUAGE    = 5
        PARODY      = 6
        TAG         = 7

    def __init__(self, id: int, name: str, type: Types, url: str, count: int):
        self._id = id
        self._type = type
        self._name = name
        self._url = url
        self._count = count

    @property
    def id(self) -> int:
        """Get id of Tag
        Retuns:
            int: id of Tag
        """

        return self._id

    @property
    def type(self) -> Types:
        """Get Type of Tag
        Retuns:
            Types: Type of Tag (e.g. language)
        """

        return self._type

    @property
    def name(self) -> str:
        """Get Name of Tag
        Retuns:
            str: Name of Tag (e.g. english)
        """

        return self._name

    @property
    def count(self) -> int:
        """Get Count of Tag
        Retuns:
            int: Count of how often used
        """

        return self._count



__version__tuple__ = ("0", "1", "1")
__author__ = ("b3yc0d3")
__email__ = ("b3yc0d3@dnmx.org")

__version__ = ".".join(__version__tuple__) # xx.xx.xx
__version_short__ = f"{__version__tuple__[0]}.{__version__tuple__[1]}" # xx.xx


# Variables
__base_url__ = "https://nhentai.net/"
__useragent__ = f"Mozilla/5.0 (compatible; nhpy/{__version__})"

__headers__ = {
    "User-Agent": __useragent__
}



import json
from typing import List

host = "https://nhentai.net"
host_images = ""
host_thumbnails = ""

class ApiPaths():

    @staticmethod
    def host_url() -> str:
        return "https://nhentai.net"

    @staticmethod
    def host_image_url() -> str:
        return "https://nhentai.net"

    @staticmethod
    def host_thumbnail_url() -> str:
        return "https://nhentai.net"

    @staticmethod
    def book_page(media_id: int, page_num: int, extension: str) -> str:
        return f"{host}/api/galleries/{media_id}/page_num.{extension}"

    @staticmethod
    def book_cover(media_id: int, extension: str) -> str:
        return f"{host}/api/galleries/{media_id}/cover.{extension}"



import requests
from typing import List
from .nbook import NBook, NTag, NImage, NPage
from .api_paths import ApiPaths
from .__vars__ import __headers__

def from_json(data: dict) -> NBook:
    media_id = data["media_id"]
    normalise = {"j": "jpg", "p": "png"}
    normalise_ttypes = {
        "artist": NTag.Types.ARTIST,
        "category": NTag.Types.CATEGORY,
        "character": NTag.Types.CHARACTER,
        "group": NTag.Types.GROUP,
        "language": NTag.Types.LANGUAGE,
        "parody": NTag.Types.PARODY,
        "tag": NTag.Types.TAG,
    }
    lnpages = []
    tags = []

    # add Cover
    cover_type = normalise[data["images"]["cover"]["t"]]
    url = ApiPaths.book_cover(media_id, cover_type)
    cover = NImage(url, cover_type, data["images"]["cover"]["w"], data["images"]["cover"]["h"])

    # add Thumbnail
    cover_type = normalise[data["images"]["thumbnail"]["t"]]
    url = ApiPaths.book_cover(media_id, cover_type)
    thumbnail = NImage(url, cover_type, data["images"]["thumbnail"]["w"], data["images"]["thumbnail"]["h"])

    # Add all Pages
    for i in range(len(data["images"]["pages"])):
        cur_page = data["images"]["pages"][i]
        page_num = i + 1
        img_type = normalise[cur_page["t"]]

        url = ApiPaths.book_page(media_id, page_num, img_type)
        tmp_img = NImage(url, img_type, cur_page["w"], cur_page["h"])

        lnpages.append(NPage(page_num, tmp_img))

    # add tages
    for it in range(len(data["tags"])):
        cur_tag = data["tags"][it]
        tags.append(NTag(cur_tag["id"], cur_tag["name"], normalise_ttypes[cur_tag["type"]], cur_tag["url"], cur_tag["count"]))


    return NBook(data["id"], media_id, data["title"], data["upload_date"], tags, data["num_favorites"], lnpages, data["num_pages"], cover, thumbnail, data["scanlator"])


class NhPy:
    def __init__(self):
        pass

    def get_book(self, id: int) -> NBook:
        """Get Book By Id
        Args:
        """
        print(ApiPaths.host_url())
        err, data = self._make_request(ApiPaths.host_url() + f"/api/gallery/{str(id)}")
        if (err):
            return None

        return from_json(data)

    def get_related_books(self, id: int) -> list:
        err, data = self._make_request(f"{ApiPaths.host_url()}/api/gallery/{str(id)}/related")
        if (err):
            return dict()

        ret_data = []

        for book in data["result"]:
            ret_data.append(from_json(book))

        return ret_data

    def search(self, query: list, page: int = 1) -> dict:
        err, data = self._make_request(f"{ApiPaths.host_url()}/api/galleries/search?query=" + "+".join(query) + "&page={page}")
        if (err):
            return None

        ret_data = {
            "pages": data["num_pages"],
            "per_page": data["per_page"],
            "books": []
        }

        for book in data["result"]:
            ret_data["books"].append(from_json(book))

        return ret_data



    def _make_request(self, url) -> list:
        response = requests.get(url, headers=__headers__)
        if (response.status_code != 200):
            return [True, dict()]

        return [False, response.json()]



class NImage():
    def __init__(self, url: str, type: str, width: int, height:int):
        """nhentai Book Image
        Args:
            url (str): url of image
            type (str): image type (e.g. png)
        """
        self._type      = type # png, jpg/jpeg
        self._url       = url
        self._width     = 0
        self._height    = 0
        self._raw       = b''

    @property
    def url(self) -> str:
        """URL of Image
        Returns:
            str: Image URL
        """
        return self._type

class NPage():
    def __init__(self, number: int, image: NImage):
        self._number = number
        self._image  = image

    @property
    def number(self) -> int:
        """Get Page Number
        Returns:
            int: Number of current Page
        """

        return self._number

    @property
    def image(self) -> NImage:
        """Get Image of Page
        Returns:
            NImage: Page Image
        """
        return self._image

class NPages():
    def __init__(self):
        self.pages = []
        self._num_pages = 0

    def get(self, page_num: int) -> NPage:
        if page_num > len(self.pages):
            return None

        return self.pages[page_num]

    @property
    def num_pages(self) -> int:
        """Get number of pages
        Returns:
            int: Number of pages
        """

        return self._num_pages



from datetime import datetime
from typing import List
import json

from .npages import NPage, NImage
from .ntag import NTag
from .api_paths import ApiPaths

class NBook(object):
    def __init__(self, id: int, media_id: str, title: List[str], upload_date: int, tags: list[NTag], favs: int, pages: List[NPage], page_count: int, cover: NImage, thumbnail: NImage, scanlator: str):
        self._id            = id
        self._media_id      = media_id
        self._title         = title
        self._upload_date   = upload_date
        self._tags          = tags
        self._favs          = favs

        self._scanlator     = scanlator #currently unsed

        self._pages         = pages
        self._page_count    = page_count
        self._cover         = cover
        self._thumbnail     = thumbnail


    @property
    def id(self) -> int:
        """Get Book Id
        Returns:
            int: Book Id
        """
        return self._id

    @property
    def title(self) -> str:
        """Get all Book Titles
        Returns:
            list[str]: List of all Book Titles
        """
        return self._title

    def uploaded(self, raw: bool = False) -> int | str:
        """Get Upload Time of Book
        Args:
            raw (bool): If should be formated or raw UNIX Time Epoch should be retuned

        Returns:
            int|str: Formated Timestamp or raw UNIX Time Epoch
        """

        if raw == True:
            return datetime.utcfromtimestamp(self._upload_date).strftime("%d %b, %Y %H:%M:%S")

        return self._upload_date

    @property
    def tags(self) -> List[NTag]:
        """Get Tags of Book
        Returns:
            list[NTags]: List of Tags
        """

        return self._tags

    @property
    def fav_count(self) -> int:
        """Get how many People markt book as Favorite
        Returns:
            int: Favorites count
        """
        return self._favs

    @property
    def pages(self) -> List[NPage]:
        """Get all Pages of Book
        Returns:
            list[NPage]: Pages
        """
        return self._pages

    @property
    def page_count(self) -> int:
        """Get number of Pages
        Returns:
            int: Number of Pages
        """

        return self._page_count

    @property
    def cover(self) -> NImage:
        """Get Cover of Book
        Returns:
            NImage: Cover Image
        """

        return self._cover

    @property
    def thumbnail(self) -> NImage:
        """Get Thumbnail of Book
        Returns:
            NImage: Thumbnail Image
        """
        return self._thumbnail



from setuptools import setup
from nhpy.__vars__ import __version__, __author__, __email__

with open('README.md', 'r') as rmdf:
    long_description = rmdf.read()

setup(
    name='nhpy',
    version=__version__,
    description='API wraper for nhentai.net',
    url="https://github.com/b3yc0d3/nhpy",
    author=__author__,
    author_email=__email__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['nhpy'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent"
    ],
    install_requires = [
        "requests >= 2.27.1"
        # "beautifulsoup4 >= 4.10.0",
    ],
    project_urls={
        "Source": "https://github.com/b3yc0d3/nhpy",
        "Issue tracker": "https://github.com/b3yc0d3/nhpy/issues"
    }
)
